import io, os, sys, time, pygame
import photoboite.buttons

try:
    import picamera

    class Camera:
        def __init__(self, rect):
            self.rect = rect
            self.camera = picamera.PiCamera()
            self.camera.led = False
            self.camera.resolution = (self.rect.width, self.rect.height)
            self.rgb = bytearray(self.rect.width * self.rect.height * 4)

        def close(self):
            self.camera.close()

        def take(self, name):
            name = name + '.jpg'
            self.camera.resolution = (3280, 2464)
            self.camera.capture(name)
            self.camera.resolution = (self.rect.width, self.rect.height)
            return name

        def stream(self):
            stream = io.BytesIO()
            self.camera.capture(stream, use_video_port=True, format='rgba', resize=(self.rect.width, self.rect.height))
            stream.seek(0)
            stream.readinto(self.rgb)
            stream.close()
            img = pygame.image.frombuffer(self.rgb, (self.rect.width, self.rect.height), 'RGBA')

            return pygame.transform.flip(img, True, False)

        def draw_on(self, screen):
            screen.blit(self.stream(), self.rect)


except:
    class Camera:
        def __init__(self, rect):
            self.rect = rect
            self.img = pygame.transform.scale(pygame.image.load(os.path.dirname(os.path.realpath(__file__)) + '/../images/tournesol.jpg'), (self.rect.width, self.rect.height))

        def close(self):
            pass

        def take(self, name):
            print('Take picture! ' + name)
            return os.path.dirname(os.path.realpath(__file__)) + '/../images/tournesol.jpg'

        def draw_on(self, screen):
            screen.blit(self.img, self.rect)



class Capture:
    def __init__(self, uid):
        self.uid = uid
        self.photos = []

    def take(self, camera):
        self.photos.append(Photo(self.uid, len(self.photos)).take(camera))


class Photo:
    def __init__(self, cid, uid):
        self.cid = cid
        self.uid = uid

    def take(self, camera):
        self.name = "photos/%s-%04d-%04d" % (time.strftime('%Y%m%d-%H%M%S'), self.cid, self.uid)
        self.capture = pygame.transform.smoothscale(pygame.image.load(camera.take(self.name)), (638, 480))

        return self


class SleepMode:
    def __init__(self, seconds, screen):
        self.enabled = True
        self.drawn = False
        self.seconds = seconds

        self.screen = screen

        self.clicker = pygame.image.load(os.path.dirname(os.path.realpath(__file__)) + '/../images/clicker.png').convert_alpha()
        self.clicker_rect = self.clicker.get_rect()
        self.clicker_rect.topleft = ((self.screen.get_width() - self.clicker_rect.width) / 2, (self.screen.get_height() - self.clicker_rect.height) / 2)

    def move(self):
        self.last_action = time.time()

    def event(self):
        for event in pygame.event.get():
            if event.type == pygame.MOUSEBUTTONUP or event.type == pygame.KEYUP:
                self.move()
                self.enabled = False

    def draw(self):
        if not self.drawn:
            self.screen.fill((255,255,255))
            self.screen.blit(self.clicker, self.clicker_rect)
            pygame.display.flip()
            self.drawn = True

    def sleep(self):
        if time.time() - self.last_action > self.seconds:
            self.enabled = True
            self.drawn = False

class Photoboite:
    def __init__(self):
        self.screen = self.screen()
        pygame.font.init()

        pygame.mouse.set_visible(False)

        self.buttons = []

        button_size = int(self.screen.get_height() / 4)
        topleft = (self.screen.get_width() - button_size) / 2, self.screen.get_height() - button_size - 10

        self.capture_button = photoboite.buttons.CaptureButton(button_size, topleft, self)
        self.buttons.append(self.capture_button)
        self.buttons.append(photoboite.buttons.PowerButton(pygame.Rect(self.screen.get_width() - 50, 0, 50, 50)))
        self.buttons.append(photoboite.buttons.InfoButton(pygame.Rect(0, 0, 50, 50), self))

        camera_rect = pygame.Rect(0, 0, 800, 480)
        self.camera = Camera(camera_rect)

        self.clock = pygame.time.Clock()

        self.sleep = SleepMode(30, self.screen)


    def screen(self):
        if os.getenv('DISPLAY'):
            pygame.display.init()
            display = pygame.display.Info()
            return pygame.display.set_mode((800, 480))
        else:
            drivers = ['directfb', 'fbcon', 'svgalib']
            found = False
            for driver in drivers:
                if not os.getenv('SDL_VIDEODRIVER'):
                    os.putenv('SDL_VIDEODRIVER', driver)
                try:
                    pygame.display.init()
                except pygame.error as e:
                    print('Driver: {0} failed.'.format(driver))
                    continue
                found = True
                break

            if not found:
               raise Exception('No suitable video driver found!')

            return pygame.display.set_mode()

    def draw(self):
        self.clock.tick(60)

        self.camera.draw_on(self.screen)

        for button in self.buttons:
            button.draw_on(self.screen)

    def event(self):
        for event in pygame.event.get():
            self.sleep.move()

            if event.type == pygame.QUIT or event.type == pygame.KEYDOWN and event.unicode == 'q':
                self.quit()
            elif event.type == pygame.MOUSEBUTTONDOWN:
                for button in self.buttons:
                    if button.is_pressed(event.pos):
                        button.press()
            elif event.type == pygame.MOUSEBUTTONUP:
                for button in self.buttons:
                    if button.is_pressed(event.pos):
                        button.unpress()

    def run(self):
        pressed = False
        while True:
            if self.sleep.enabled:
                self.sleep.event()
                self.sleep.draw()
                self.clock.tick(60)
            else:
                self.event()
                self.draw()
                self.sleep.sleep()
                pygame.display.flip()

    def quit(self):
        self.camera.close()
        sys.exit()
